use ::app::get_router;
use std::env;
use std::path::Path;
use listenfd::ListenFd;
use tokio::net::TcpListener;

#[tokio::main]
async fn main() {
    let manifest_dir = env::var("CARGO_MANIFEST_DIR").unwrap();
    let assets_path = Path::new(&manifest_dir).join("assets");
    let router = get_router(assets_path.as_path());

    let mut listenfd = ListenFd::from_env();
    let listener = match listenfd.take_tcp_listener(0).unwrap() {
        // if we are given a tcp listener on listen fd 0, we use that one
        Some(listener) => {
            listener.set_nonblocking(true).unwrap();
            TcpListener::from_std(listener).unwrap()
        }
        // otherwise fall back to local listening
        None => TcpListener::bind("127.0.0.1:3000").await.unwrap(),
    };

    println!("Listening on: http://{}", listener.local_addr().unwrap());

    // Run the server with the router
    axum::serve(listener, router.into_make_service()).await.unwrap();
}
